use std::sync::Arc;

use actix_web::{get, web, HttpResponse, Responder};
use poise::serenity_prelude::Http;
use serde::{Deserialize, Serialize};
use utoipa::ToSchema;

use crate::config::Config;

#[derive(Serialize, Deserialize, ToSchema)]
pub struct InfoGuild {
    pub name: String,
    pub id: u64,
}

#[derive(Serialize, Deserialize, ToSchema)]
pub struct Info {
    pub name: String,
    pub version: String,
    pub description: String,
    pub available_guids: Vec<InfoGuild>,
}

#[utoipa::path(
    tag= "Bot",
    operation_id = "getInfo",
    path = "/api/bot/info",
    responses(
        (status = 200, description = "Information about the bot", body = Info),
        (status = 500, description = "Internal server error")
    )
)]
#[get("/info")]
#[tracing::instrument(name = "get_info", skip(config, http))]
pub async fn get_info(config: web::Data<Config>, http: web::Data<Arc<Http>>) -> impl Responder {
    let list_guilds = http.get_guilds(None, None).await;
    match list_guilds {
        Ok(guilds) => HttpResponse::Ok().json(Info {
            name: "BotDiscord".to_string(),
            version: env!("CARGO_PKG_VERSION").to_string(),
            description: config.bot_name.clone(),
            available_guids: guilds
                .iter()
                .map(|g| InfoGuild {
                    name: g.name.clone(),
                    id: g.id.get(),
                })
                .collect(),
        }),
        Err(e) => HttpResponse::InternalServerError().body(format!("Error getting guilds: {}", e)),
    }
}
