use crate::botv2::{
    domain::concour::{
        add_keyword::{add_keyword_concour, AddKeywordConcourError, KeyWordSource},
        check_if_allowed::check_if_allowed,
    },
    init::{Context, Error},
};
use poise::{
    serenity_prelude::{model::colour, CreateEmbed, CreateEmbedFooter, Mentionable, RoleId},
    CreateReply,
};
use tracing::instrument;

/// Add keyword for concour (only for admin)
/// this function will turn into submodule in the futur
#[instrument(skip(ctx), level = "info", fields(channel = ctx.channel_id().get(), guild = ?ctx.guild_id().unwrap().get()))]
#[poise::command(
    slash_command,
    prefix_command,
    category = "server_config",
    guild_only = true
)]
pub async fn keyword(
    ctx: Context<'_>,
    #[description = "Delimiter de list (\\n par defaut)"] delimiter: Option<String>,
    #[description = "Url de la liste"] url: String,
) -> Result<(), Error> {
    let guild = match ctx.guild_id() {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let entity_name = ctx.data().entity_name.clone();
    let footer = CreateEmbedFooter::new(entity_name.clone());
    match check_if_allowed(guild.get(), ctx.author().id.get(), ctx.http()).await {
        Ok(ok) => {
            if !ok {
                let embed = CreateEmbed::new()
                    .title("You are not an admin")
                    .color(colour::Color::RED)
                    .footer(footer);
                if let Err(why) = ctx
                    .send(CreateReply::default().embed(embed).ephemeral(true))
                    .await
                {
                    tracing::error!("Error sending message: {:?}", why);
                }
                return Ok(());
            }
        }
        Err(_) => {
            let embed = CreateEmbed::new()
                .title("You are not an admin")
                .color(colour::Color::RED)
                .footer(footer);
            if let Err(why) = ctx
                .send(CreateReply::default().embed(embed).ephemeral(true))
                .await
            {
                tracing::error!("Error sending message: {:?}", why);
            }
            return Ok(());
        }
    };
    let concour = match add_keyword_concour(
        guild.get(),
        ctx.channel_id().get(),
        KeyWordSource::Url(url, delimiter),
    )
    .await
    {
        Ok(concour) => {
            if concour.is_none() {
                CreateEmbed::new()
                    .title("No concour updated")
                    .color(colour::Color::RED)
            } else {
                let concour = concour.unwrap();
                let output = CreateEmbed::new()
                    .title(concour.title)
                    .description(concour.description)
                    .field("Start date", concour.start_date.to_string(), false)
                    .field("Periode", concour.periode.to_string(), false)
                    .field("keyword", concour.keywords.len().to_string(), true);

                if concour.role_recompense != 0 {
                    output.field(
                        "Role récompense",
                        RoleId::new(concour.role_recompense).mention().to_string(),
                        false,
                    )
                } else {
                    output
                }
            }
        }
        Err(err) => match err {
            AddKeywordConcourError::DoesntExist => CreateEmbed::new()
                .title("Concour dont exist")
                .color(colour::Color::RED),
            _ => CreateEmbed::new()
                .title("Error while creating concour")
                .field("Please contact your administrator", "", false)
                .field("Your concour is possibly not initied correctly", "", false)
                .color(colour::Color::RED),
        },
    };

    let mut builder = CreateReply::default().ephemeral(true);
    builder = builder.embed(concour.footer(footer));
    if let Err(why) = ctx.send(builder).await {
        tracing::error!("Error sending message: {:?}", why);
    }
    Ok(())
}
