use poise::serenity_prelude;
use serde::{Deserialize, Serialize};
use tracing::instrument;

use crate::{
    botv2::domain::server_config::{
        check_if_server_enable::check_if_server_enable,
        check_if_server_enable_and_admin::check_if_server_enable_and_user_admin,
    },
    db::server_config::ServerConfig,
};

#[derive(Serialize, Deserialize, Debug, Clone)]
pub enum CheckIfConcourEnable {
    UnknownError(String),
}

// Check :
// - if the user is admin or owner of the server
// - if the server enabled the concour feature
// - if the user has a role allowed to handle concour

/// Check if the concour is allowed on the server
#[instrument(level = "info")]
pub async fn check_if_concour_allowed(
    server_id: u64,
) -> Result<(bool, Option<ServerConfig>), CheckIfConcourEnable> {
    if let Ok((enable, server_config)) = check_if_server_enable(server_id).await {
        if !enable {
            return Ok((false, None));
        }
        if let Some(server_config) = server_config {
            return Ok((server_config.auto_concour, Some(server_config)));
        }
    }
    Ok((false, None))
}

/// Check if the concour is allowed on the server and if the user has a role allowed to handle concour
#[instrument(level = "info", skip(http))]
pub async fn check_if_allowed(
    server_id: u64,
    user_id: u64,
    http: &serenity_prelude::Http,
) -> Result<bool, CheckIfConcourEnable> {
    if let Ok((enable, server_config)) =
        check_if_server_enable_and_user_admin(server_id, user_id, http).await
    {
        if !enable {
            return Ok(false);
        }
        if let Some(server_config) = server_config {
            return Ok(server_config.auto_concour);
        }
    }
    Ok(false)
}
