use super::init::DB;
use serde::{Deserialize, Serialize};
use std::fmt;
use surrealdb::opt::Resource;
use tracing::instrument;
use utoipa::ToSchema;

const SERVER_CONFIG: &str = "server_config";
const fn default_false() -> bool {
    false
}

#[derive(Debug, Serialize, Deserialize, Clone, ToSchema)]
pub struct ServerConfig {
    #[serde(default = "default_false")]
    pub enable: bool,
    pub server_id: u64,
    #[serde(default = "default_false")]
    pub auto_meme: bool,
    #[serde(default = "default_false")]
    pub auto_concour: bool,
    pub admin_role: Vec<u64>,
}

impl fmt::Display for ServerConfig {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "{:?}", self)
    }
}

impl ServerConfig {
    pub fn new(server_id: u64) -> Result<Self, surrealdb::Error> {
        Ok(Self {
            enable: false,
            server_id,
            auto_meme: false,
            auto_concour: false,
            admin_role: Vec::new(),
        })
    }
    #[instrument(level = "info")]
    pub async fn create(&self) -> Result<(), surrealdb::Error> {
        match DB
            .create(Resource::from(SERVER_CONFIG))
            .content(&self)
            .await
        {
            Ok(_) => Ok(()),
            Err(e) => Err(e),
        }
    }
    #[instrument(level = "info")]
    pub async fn update(&self) -> Result<(), surrealdb::Error> {
        let sql = format!(
            "UPDATE {} SET enable = {}, auto_meme = {}, auto_concour = {}, admin_role = {:?} WHERE server_id = {}",
            SERVER_CONFIG,
            self.enable,
            self.auto_meme,
            self.auto_concour,
            self.admin_role,
            self.server_id
        );
        match DB.query(&sql).await {
            Ok(_) => Ok(()),
            Err(e) => Err(e),
        }
    }
    #[instrument(level = "info")]
    pub async fn find_by_server_id(
        server_id: &u64,
    ) -> Result<Option<ServerConfig>, surrealdb::Error> {
        let sql = format!(
            "SELECT * FROM {} WHERE server_id = {}",
            SERVER_CONFIG, server_id
        );
        let mut results = match DB.query(&sql).await {
            Ok(results) => results,
            Err(e) => {
                return Err(e);
            }
        };
        let server_config: ServerConfig = match results.take(0) {
            Ok(Some(server_config)) => server_config,
            Ok(None) => {
                return Ok(None);
            }
            Err(e) => {
                return Err(e);
            }
        };

        Ok(Some(server_config))
    }
}
